from gemini_model.model_abstract import Model
from typing_extensions import List, TypedDict
from langchain_community.document_loaders import PyPDFLoader, TextLoader
from langchain_core.documents import Document
import ollama

from pydantic import BaseModel, Field

import os
import re


class CitedAnswer(BaseModel):
    """Answer the user question based only on the given sources, and cite the sources used."""

    answer: str = Field(
        ...,
        description="The answer to the user question, which is based only on the given sources.",
    )
    citations: List[int] = Field(
        ...,
        description="The integer IDs of the SPECIFIC sources which justify the answer.",
    )


# Define state for RAG application
class State(TypedDict):
    question: str
    context: List[Document]
    answer: CitedAnswer


class RAG(Model):
    def __init__(self):
        super().__init__()
        """ Model initialization
        """
        self.parameters = {}
        self.output = {}
        self.text_splitter = None

    def load_files(self, docs_dir, filter_files=None):
        documents = []
        for file_name in os.listdir(docs_dir):
            if filter_files and file_name not in filter_files:
                continue

            file_path = os.path.join(docs_dir, file_name)
            if file_name.endswith(".pdf"):
                loader = PyPDFLoader(file_path)
                documents.extend(loader.load())
            elif file_name.endswith(".txt"):
                loader = TextLoader(file_path, encoding="utf-8")
                documents.extend(loader.load())
        return documents

    def readfiles(self, docs_dir, filenames=None):
        text_contents = {}

        # Use provided filenames or list all from directory
        if filenames is None:
            filenames = os.listdir(docs_dir)

        for filename in filenames:
            file_path = os.path.join(docs_dir, filename)

            if not os.path.isfile(file_path):
                continue

            if filename.endswith(".txt"):
                with open(file_path, "r", encoding="utf-8") as file:
                    content = file.read()
                text_contents[filename] = content

            elif filename.endswith(".pdf"):
                loader = PyPDFLoader(file_path)
                documents = loader.load()
                content = "\n".join(doc.page_content for doc in documents)
                text_contents[filename] = content

        return text_contents

    def chunksplitter(self, text, chunk_size):
        words = re.findall(r'\S+', text)

        chunks = []
        current_chunk = []
        word_count = 0

        for word in words:
            current_chunk.append(word)
            word_count += 1

            if word_count >= chunk_size:
                chunks.append(' '.join(current_chunk))
                current_chunk = []
                word_count = 0

        if current_chunk:
            chunks.append(' '.join(current_chunk))

        return chunks

    def getembedding(self, chunks):
        # Model options are "mxbai-embed-large", "nomic-embed-text", "all-minilm"
        embeds = ollama.embed(model="nomic-embed-text", input=chunks)
        return embeds.get('embeddings', [])

    def format_docs_with_id(self, docs: List[Document]) -> str:
        formatted = [
            f"Source ID: {i}\nSource document: {doc.metadata['source']}\nSource snippet: {doc.page_content}"
            for i, doc in enumerate(docs)
        ]
        return "\n\n" + "\n\n".join(formatted)


    def update_parameters(self, parameters):
        pass


    def initialize_state(self, x):
        pass


    def update_state(self, u, x):
        pass


    def calculate_output(self, u, x):
        pass

    def get_output(self):
        pass
